local KhioHud = LibStub("AceAddon-3.0"):GetAddon("KhioHud")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("KhioHud")

local KH = _G["KhioHud"]

KhioHud.OptionsTable = {
	type = "group",
	args = {
		header = {
			name = L["In this section you can change the settings of Khio Hud.\n\n"],
			type = "description",
			order = 0,
			width = "full",
		},
		general = {
			name = L["General"],
			type = "group",
			order = 1,
			args = {
				autoToggleInside = {
					name = L["Enable automagically inside istances"],
					type = "toggle",
					order = 0,
					width = "full",
					set = function(info, value) KhioHudDB.autoToggle.inside = value end,
					get = function(info) return KhioHudDB.autoToggle.inside end,
				},
				autoToggleOutside = {
					name = L["Disable automagically outside istances"],
					type = "toggle",
					order = 1,
					width = "full",
					set = function(info, value) KhioHudDB.autoToggle.outside = value end,
					get = function(info) return KhioHudDB.autoToggle.outside end,
				},
			},
		},
		c1 = {
			name = L["Circle"] .. " #1",
			type = "group",
			order = 2,
			args = {
				radius = {
					name = L["Radius"],
					type = "range",
					order = 0,
					width = "full",
					min = 1,
					max = 50,
					step = 1,
					set = function (info, value) KhioHudDB.c1.radius = value KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c1.radius end,
				},
				color = {
					name = L["Color"],
					type = "color",
					order = 1,
					width = "full",
					hasAlpha = true,
					set = function (info, rr, gg, bb, aa) KhioHudDB.c1.color = { r = rr, b = bb, g = gg, a = aa } KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c1.color.r, KhioHudDB.c1.color.g, KhioHudDB.c1.color.b, KhioHudDB.c1.color.a end,
				},
			},
		},
		c2 = {
			name = L["Circle"] .. " #2",
			type = "group",
			order = 3,
			args = {
				radius = {
					name = L["Radius"],
					type = "range",
					order = 0,
					width = "full",
					min = 1,
					max = 50,
					step = 1,
					set = function (info, value) KhioHudDB.c2.radius = value KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c2.radius end,
				},
				color = {
					name = L["Color"],
					type = "color",
					order = 1,
					width = "full",
					hasAlpha = true,
					set = function (info, rr, gg, bb, aa) KhioHudDB.c2.color = { r = rr, b = bb, g = gg, a = aa } KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c2.color.r, KhioHudDB.c2.color.g, KhioHudDB.c2.color.b, KhioHudDB.c2.color.a end,
				},
			},
		},
		c3 = {
			name = L["Circle"] .. " #3",
			type = "group",
			order = 4,
			args = {
				radius = {
					name = L["Radius"],
					type = "range",
					order = 0,
					width = "full",
					min = 1,
					max = 50,
					step = 1,
					set = function (info, value) KhioHudDB.c3.radius = value KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c3.radius end,
				},
				color = {
					name = L["Color"],
					type = "color",
					order = 1,
					width = "full",
					hasAlpha = true,
					set = function (info, rr, gg, bb, aa) KhioHudDB.c3.color = { r = rr, b = bb, g = gg, a = aa } KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.c3.color.r, KhioHudDB.c3.color.g, KhioHudDB.c3.color.b, KhioHudDB.c3.color.a end,
				},
			},
		},
		cone = {
			name = L["Cone"],
			type = "group",
			order = 5,
			args = {
				radius = {
					name = L["Radius"],
					type = "range",
					order = 0,
					width = "full",
					min = 1,
					max = 50,
					step = 1,
					set = function (info, value) KhioHudDB.cone.radius = value KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.cone.radius end,
				},
				color = {
					name = L["Color"],
					type = "color",
					order = 1,
					width = "full",
					hasAlpha = true,
					set = function (info, rr, gg, bb, aa) KhioHudDB.cone.color = { r = rr, b = bb, g = gg, a = aa } KhioHud:UpdateSettings() end,
					get = function (info) return KhioHudDB.cone.color.r, KhioHudDB.cone.color.g, KhioHudDB.cone.color.b, KhioHudDB.cone.color.a end,
				},
			},
		},
	},
}

function KhioHud:UpdateSettings()
	for i = 1, 3 do
		local circle = KH.minimapOverlay["circle" .. i]
		local yards = self:YardsToSize(KhioHudDB["c" .. i].radius)
		circle:SetSize(yards, yards)
		circle:SetVertexColor(KhioHudDB["c" .. i].color.r, KhioHudDB["c" .. i].color.g, KhioHudDB["c" .. i].color.b)
		circle:SetAlpha(KhioHudDB["c" .. i].color.a)
	end

	local cone = KH.minimapOverlay.cone
	local yards = self:YardsToSize(KhioHudDB.cone.radius)
	cone:SetSize(yards, yards)
	cone:SetVertexColor(KhioHudDB.cone.color.r, KhioHudDB.cone.color.g, KhioHudDB.cone.color.b)
	cone:SetAlpha(KhioHudDB.cone.color.a)
end

function KhioHud:InitializeSettings()
	KhioHudDB = KhioHudDB or {
		autoToggle = {
			inside = false,
			outside = false,
		},
		c1 = {
			radius = 40,
			color = { r = 1, g = 1, b = 1, a = 0.2 },
		},
		c2 = {
			radius = 15,
			color = { r = 1, g = 1, b = 1, a = 0.2 },
		},
		c3 = {
			radius = 5,
			color = { r = 1, g = 1, b = 1, a = 0.2 },
		},
		cone = {
			radius = 40,
			color = { r = 1, g = 1, b = 1, a = 0.2 },
		},
		minimap = {
			x = nil,
			y = nil
		},
	}

	self:UpdateSettings()
end

function KhioHud:AddToBlizOptions()
	AceConfig:RegisterOptionsTable("Khio Hud", KhioHud.OptionsTable)
	AceConfigDialog:AddToBlizOptions("Khio Hud", "Khio Hud", nil)
end

function KhioHud:YardsToSize(yards)
	local magicNumber = 115 -- Estimated size for 40 yards spells
	return (magicNumber * yards) / 40
end
